/*	$Id: imalloc.c 2007/11/26 11:28:16 mallet $ */

/* 
 * Copyright (c) 1997 LAAS/CNRS                       --  Thu Aug  3 2006
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

#include <stdlib.h>
#include <string.h>
#include <float.h>

#include "libimages3d.h"

/**********************************************************************
 *
 * Simple allocation d'une structure 'DATA_IM3D' (le tableau de points n'est
 * pas allou)
 *
 **********************************************************************/

int
cree_data_im3d(DATA_IM3D **im3d,
	       int nblig, int nbcol,
	       int covar, int normales, int size_local)
{
  DATA_IM3D*    new_im3d;

  new_im3d = (DATA_IM3D*) malloc (sizeof (DATA_IM3D));

  if (new_im3d == NULL)
    return (ERR_IM3D_MALLOC);
  else
    *im3d = new_im3d;

  return init_data_im3d(*im3d, nblig,nbcol,
			covar, normales, size_local);
}

int
init_data_im3d(DATA_IM3D *im3d,
	       int nblig, int nbcol,
	       int covar, int normales, int size_local)
{
  int           i;		/* Pour boucler */

  /* On alloue la tr3d tr_capteur_2_repere */
  im3d->header.tr_capteur_2_repere = t3dMatrixIdentity;

  /* On ne l'initialise pas, il n'y a pas encore de points dtermins */

  /* On positionne les flags comme il faut */
  im3d->header.robot = NIL_ROBOT;
  im3d->header.capteur = NIL_CAPTEUR;
  im3d->header.repere = NIL_REPERE;
  im3d->header.coordonnees = NIL_COORDONNEES;
  im3d->header.points = NIL_POINTS;
  im3d->header.normales = NIL_NORMALES;
  im3d->header.nb_voisins_normales = NIL_TYPE_NORMALES;
  im3d->header.covariances = NIL_COVARIANCES;
  im3d->header.local = NIL_INFOS_LOCALES;
  im3d->header.bornes = NIL_BORNES;
  im3d->header.reduction = NIL_REDUCTION;
  im3d->header.sous_image = NIL_SOUS_IMAGE;

  im3d->header.size_local = 0;

  im3d->header.taux_reduction = 1;
  im3d->header.nblig_initial = 0;
  im3d->header.nbcol_initial = 0;
  im3d->header.nblig = 0;
  im3d->header.nbcol = 0;
  im3d->header.i_min = 0;
  im3d->header.i_max = 0;
  im3d->header.j_min = 0;
  im3d->header.j_max = 0;

  /* Quelques initialisations */
  im3d->header.max_coord_1 = - FLT_MAX;
  im3d->header.min_coord_1 = FLT_MAX;
  im3d->header.max_coord_2 = - FLT_MAX;
  im3d->header.min_coord_2 = FLT_MAX;
  im3d->header.max_coord_3 = - FLT_MAX;
  im3d->header.min_coord_3 = FLT_MAX;

  im3d->points_tab = NULL;
  im3d->covariances_tab = NULL;
  im3d->normales_tab = NULL;
  im3d->local_tab = NULL;

  for (i = 0; i < 16; i++)
    im3d->header.mat_intrinseque[i] = 0.0;

  if (nblig && nbcol &&
      alloue_im3d_points(im3d, nblig, nbcol) != ERR_IM3D_OK)
     return ERR_IM3D_MALLOC_PT3D;

  if (covar && alloue_im3d_covariances(im3d) != ERR_IM3D_OK)
     return ERR_IM3D_MALLOC_PT3D_COVARIANCES;

  if (normales && alloue_im3d_normales(im3d) != ERR_IM3D_OK)
     return ERR_IM3D_MALLOC_PT3D_NORMALES;

  if (size_local &&
      alloue_im3d_infos_locales(im3d, size_local) != ERR_IM3D_OK)
     return ERR_IM3D_MALLOC_PT3D_INFOS_LOCALES;

  return (ERR_IM3D_OK);
}

/**********************************************************************
 *
 * Allocation du tableau de points d'une structure 'DATA_IM3D'
 *
 **********************************************************************/

int alloue_im3d_points(DATA_IM3D*  im3d,
			     int         nblig,
			     int         nbcol)
{
  /* validit des indices */
  if ((nblig <= 0) || (nbcol <= 0))
    return (ERR_IM3D_TAILLE);

  /* On vrifie que le tableau des points n'est pas dj allou */
  if (im3d->header.points != NIL_POINTS)
     free (im3d->points_tab);

  /* Allocation du tableau de points */
  im3d->points_tab = (DATA_PT3D*) malloc (nblig * nbcol * sizeof (DATA_PT3D));

  if (im3d->points_tab == NULL)
    return (ERR_IM3D_MALLOC_POINTS);

  /* Mise  jour des paramtres et des flags */
  im3d->header.points = POINTS_ALLOUES;
  im3d->header.nblig = nblig;
  im3d->header.nbcol = nbcol;
  im3d->header.nblig_initial = nblig;
  im3d->header.nbcol_initial = nbcol;
  im3d->header.i_min = 0;
  im3d->header.i_max = nblig;
  im3d->header.j_min = 0;
  im3d->header.j_max = nbcol;

  /* On met tout  zro : a a l'avantage d'initialiser les flags des points */
  memset ( (DATA_PT3D*) im3d->points_tab, 0,
	   nblig * nbcol * sizeof (DATA_PT3D) );

  return (ERR_IM3D_OK);
}

/**********************************************************************
 *
 * Allocation du tableau de covariances d'une structure 'DATA_IM3D'
 *
 **********************************************************************/

int alloue_im3d_covariances(DATA_IM3D*  im3d)
{
  int         i;		/* Pour boucler sur les points */
  DATA_PT3D*  point;		/* Pointeur sur le tableau de points */
  float*      covariance;	/* Pointeur sur le tableau de covariances */

  int         nblig;
  int         nbcol;
  int         nbpoints;

  /* On verifie que les points sont dj allous... */
  if (im3d->header.points == NIL_POINTS)
    return (ERR_IM3D_POINTS_NON_ALLOUES);
  /* ... et que le tableau des covariances ne l'est pas */
  if (im3d->header.covariances != NIL_COVARIANCES)
     free (im3d->covariances_tab);

  nblig = im3d->header.nblig;
  nbcol = im3d->header.nbcol;
  nbpoints = nblig * nbcol;

  im3d->covariances_tab
    = (float*) malloc (nblig * nbcol * IM3D_COVARIANCES_SIZE * sizeof(float));

  if (im3d->covariances_tab == NULL)
    return (ERR_IM3D_MALLOC_COVARIANCES);

  /* Mise  jour des paramtres et des flags */
  im3d->header.covariances = COVARIANCES_ALLOUEES;

  /* On met tout  zro (intret ?) */
  memset ( (float*) im3d->covariances_tab, 0,
	   nblig * nbcol * IM3D_COVARIANCES_SIZE * sizeof(float) );

  /* On va boucler sur les points pour relier les pointeurs des points dans le
     tableau des covariances */
  point = im3d->points_tab;
  covariance = im3d->covariances_tab;

  for (i = 0; i < nbpoints; i++) {
    point->covariances = COVARIANCES_ALLOUEES;
    point->covariances_tab = covariance;
    point++;
    covariance += IM3D_COVARIANCES_SIZE;
  }

  return (ERR_IM3D_OK);
}

/**********************************************************************
 *
 * Allocation du tableau de normales d'une structure 'DATA_IM3D'
 *
 **********************************************************************/

int alloue_im3d_normales (DATA_IM3D*  im3d)

{
  int         i;		/* Pour boucler sur les points */
  DATA_PT3D*  point;		/* Pointeur sur le tableau de points */
  float*      normale;		/* Pointeur sur le vecteur normal */

  int         nblig;
  int         nbcol;
  int         nbpoints;

  /* On verifie que les points sont dj allous... */
  if (im3d->header.points == NIL_POINTS)
    return (ERR_IM3D_POINTS_NON_ALLOUES);
  /* ... et que le tableau des normales ne l'est pas */
  if (im3d->header.normales != NIL_NORMALES)
     free (im3d->normales_tab);

  nblig = im3d->header.nblig;
  nbcol = im3d->header.nbcol;
  nbpoints = nblig * nbcol;

  im3d->normales_tab
    = (float*) malloc (nblig * nbcol * IM3D_NORMALES_SIZE * sizeof(float));

  if (im3d->normales_tab == NULL)
    return (ERR_IM3D_MALLOC_NORMALES);

  /* Mise  jour des paramtres et des flags */
  im3d->header.normales = NORMALES_ALLOUEES;

  /* On met tout  zro (intret ?) */
  memset ( (float*) im3d->normales_tab, 0,
	   nblig * nbcol * IM3D_NORMALES_SIZE * sizeof(float) );

  /* On va boucler sur les points pour relier les pointeurs des points dans le
     tableau des normales */
  point = im3d->points_tab;
  normale = im3d->normales_tab;

  for (i = 0; i < nbpoints; i++) {
    point->normales = NORMALES_ALLOUEES;
    point->normales_tab = normale;
    point++;
    normale += IM3D_NORMALES_SIZE;
  }

  return (ERR_IM3D_OK);
}

/**********************************************************************
 *
 * Allocation du tableau des infos locales d'une structure 'DATA_IM3D'
 *
 **********************************************************************/

int alloue_im3d_infos_locales (DATA_IM3D*  im3d,
				     int         size_infos)

{
  int         i;		/* Pour boucler sur les points */
  DATA_PT3D*  point;		/* Pointeur sur le tableau de points */
  void*       info_locale;		/* Pointeur sur les infos locales */

  int         nblig;
  int         nbcol;
  int         nbpoints;

  /* On verifie que les points sont dj allous... */
  if (im3d->header.points == NIL_POINTS)
    return (ERR_IM3D_POINTS_NON_ALLOUES);
  /* ... et que le tableau des infos locales ne l'est pas */
  if (im3d->header.local != NIL_INFOS_LOCALES)
     free (im3d->local_tab);

  im3d->header.size_local = size_infos;

  nblig = im3d->header.nblig;
  nbcol = im3d->header.nbcol;
  nbpoints = nblig * nbcol;

  im3d->local_tab
    = (void*) malloc (nblig * nbcol * size_infos);

  if (im3d->local_tab == NULL)
    return (ERR_IM3D_MALLOC_INFOS_LOCALES);

  /* Mise  jour des paramtres et des flags */
  im3d->header.local = INFOS_LOCALES_ALLOUEES;

  /* On met tout  zro (intret : vite des "rui" quand on duplique les pts) */
  memset ( (void*) im3d->local_tab, 0, nblig * nbcol * size_infos );

  /* On va boucler sur les points pour relier les pointeurs des points dans le
     tableau des normales */
  point = im3d->points_tab;
  info_locale = im3d->local_tab;

  for (i = 0; i < nbpoints; i++) {
    point->local = INFOS_LOCALES_ALLOUEES;
    point->local_tab = info_locale;
    point++;
    info_locale = ((char*) info_locale) + size_infos;
    /* pas de += car : cc : warning: a cast does not yield an lvalue */
  }

  return (ERR_IM3D_OK);
}


/**********************************************************************
 *
 * Destruction d'une structure 'DATA_IM3D'
 *
 **********************************************************************/

void
del_data_im3d(DATA_IM3D**    im3d)
{
   empty_data_im3d(*im3d);

   /* On efface la structure elle-mme */
   free (*im3d);
   *im3d = NULL;	/* Pour que l'interprteur libre la place
			   dans le tableau  */
}

void
empty_data_im3d(DATA_IM3D *im3d)
{
  /* On commence par effacer les tableaux de donnes */
   if (im3d->header.points != NIL_POINTS) {
      free (im3d->points_tab);
      im3d->header.points = NIL_POINTS;
   }
   if (im3d->header.covariances != NIL_COVARIANCES) {
      free (im3d->covariances_tab);
      im3d->header.covariances = NIL_COVARIANCES;
   }
   if (im3d->header.normales != NIL_NORMALES) {
      free (im3d->normales_tab);
      im3d->header.normales = NIL_NORMALES;
   }
   if (im3d->header.local != NIL_INFOS_LOCALES) {
      free (im3d->local_tab);
      im3d->header.local = NIL_INFOS_LOCALES;
   }
}

/**********************************************************************
 *
 *
 *
 **********************************************************************/

DATA_IM3D*
copy_data_im3d(DATA_IM3D* im3d)
{
  DATA_IM3D*    new_im3d;
  int		covar = 0;
  int		normales = 0;
  int		size_local = 0;

  int		nblig, nbcol;

  nblig = im3d->header.nblig;
  nbcol = im3d->header.nbcol;

  new_im3d = (DATA_IM3D*) malloc (sizeof (DATA_IM3D));

  if (new_im3d == NULL) {
    (ERR_IM3D_MALLOC);
    return (NULL);
  }

  if (im3d->header.covariances == COVARIANCES_ALLOUEES)
    covar = 1;
  if (im3d->header.normales == NORMALES_ALLOUEES)
    normales = 1;
  if (im3d->header.local == INFOS_LOCALES_ALLOUEES)
    size_local = im3d->header.size_local;

  if (init_data_im3d(new_im3d, nblig, nbcol,
		     covar, normales, size_local) != ERR_IM3D_OK)
    return (NULL);

  if (memcpy (&(new_im3d->header), &(im3d->header),
	      sizeof (DATA_IM3D_HEADER)) == NULL)
    return (NULL);

  t3dCopyIn(&new_im3d->header.tr_capteur_2_repere,
	    &im3d->header.tr_capteur_2_repere);

  if (memcpy (new_im3d->points_tab, im3d->points_tab,
	      nblig * nbcol * sizeof (DATA_PT3D)) == NULL)
    return (NULL);

  if (covar == 1)
    if (memcpy (new_im3d->covariances_tab, im3d->covariances_tab,
		nblig * nbcol * IM3D_COVARIANCES_SIZE * sizeof (float)) == NULL)
      return (NULL);

  if (normales == 1)
    if (memcpy (new_im3d->normales_tab, im3d->normales_tab,
		nblig * nbcol * IM3D_NORMALES_SIZE * sizeof (float)) == NULL)
      return (NULL);

  if (size_local != 0)
    if (memcpy (new_im3d->local_tab, im3d->local_tab,
		nblig * nbcol * size_local) == NULL)
      return (NULL);

  return (new_im3d);
}

