/*	$Id: iminfo.c 2007/11/23 18:46:42 mallet $ */

/*
 * Copyright (c) 1997 LAAS/CNRS                       --  Thu Aug  3 2006
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

#include <math.h>

#include "libimages3d.h"

/**********************************************************************
 *
 * Affiche  l'cran tout ce qu'on sait sur l'image 3D
 *
 **********************************************************************/

int info_data_im3d (DATA_IM3D*  im3d)

{
  /* On commence par les infos sur le type de l'image */
  /* Le robot d'o vient l'image */
  switch (im3d->header.robot){
  case NIL_ROBOT:
    printf ("\t(Pas de robot dfini)\n");
    break;
  case ROBOT_ADAM:
    printf ("\tImage acquise avec Adam\n");
    break;
  case ROBOT_HILARE:
    printf ("\tImage acquise avec Hilare\n");
    break;
  case ROBOT_HILARE2:
    printf ("\tImage acquise avec Hilare2\n");
    break;
  case ROBOT_LAMA:
    printf ("\tImage acquise avec Lama\n");
    break;
  case ROBOT_EVE:
    printf ("\tImage acquise avec Eve\n");
    break;
  default:
    printf ("\tImage acquise avec un robot INCONNU !!!\n");
    break;
  }

  /* Le type de capteur utlis */
  switch (im3d->header.capteur){
  case NIL_CAPTEUR:
    printf ("\t(Pas de capteur dfini)\n");
    break;
  case CAPTEUR_LASER1:
    printf ("\tImage acquise par le tlmtre Laser d'Hilare2\n");
    break;
  case CAPTEUR_LASER2:
    printf ("\tImage acquise par le tlmtre Laser de Lama\n");
    break;
  case CAPTEUR_STEREO:
    printf ("\tImage acquise par strovision\n");
    break;
  case CAPTEUR_STEREO_GEROMS94:
    printf ("\tImage acquise par strovision (geroms94)\n");
    break;
  case CAPTEUR_LASER_SIMULATION:
    printf ("\tImage acquise avec un Laser Simul\n");
    break;
  case CAPTEUR_STEREO_SIMULATION:
    printf ("\tImage acquise avec un banc Stro Simul\n");
    break;
  default:
    printf ("\tImage acquise avec un capteur INCONNU !!!\n");
    break;
  }

  /* Le repre dans lequel sont exprims les points */
  switch (im3d->header.repere) {
  case NIL_REPERE:
    printf ("\t(Pas de repre dfini)\n");
    break;
  case REPERE_CAPTEUR:
    printf ("\tPoints exprims dans le repre du capteur\n");
    break;
  case REPERE_AUTRE:
    printf ("\tPoints exprims dans un repre diffrent du capteur\n");
    break;
  }

  /* Systme de coordonnes */
  switch (im3d->header.coordonnees) {
  case NIL_COORDONNEES:
    printf ("\t(Pas de systme de coordonnes dfini)\n");
    break;
  case COORDONNEES_CARTESIENNES:
    printf ("\tSystme de coordonnes cartsien\n");
    break;
  case COORDONNEES_CYLINDRIQUES:
    printf ("\tSystme de coordonnes cylindrique\n");
    break;
  case COORDONNEES_SPHERIQUES:
    printf ("\tSystme de coordonnes sphrique\n");
    break;
  default:
    printf ("\tSystme de coordonnes INCONNU !!!\n");
    break;
  }

  /* L'image a-t-elle t rduite ? */
  switch (im3d->header.reduction) {
  case NIL_REDUCTION:
    printf ("\t(L'image n'a pas t sous-chantillonne)\n");
    break;
  case REDUCTION:
    printf ("\tL'image a t sous-chantillonne d'un facteur %d\n",
	    im3d->header.taux_reduction);
    break;
  default:
    printf ("\tValeur du champ 'reduction' impossible !!!\n");
    break;
  }

  /* L'image a-t-elle t extraite d'une image plus grande ? */
  switch (im3d->header.sous_image) {
  case NIL_SOUS_IMAGE:
    printf ("\t(L'image n'a pas t extraite d'une image plus grande)\n");
    break;
  case SOUS_IMAGE:
    printf ("\tL'image a t extraite d'une image plus grande\n");
    printf ("\tCoordonnes de la sous-image dans l'image originale (avant rduction):\n");
    printf ("\t\t(I_min, I_max) = (%d, %d), (J_min, J_max) = (%d, %d)\n",
	    im3d->header.i_min, im3d->header.i_max,
	    im3d->header.j_min, im3d->header.j_max);
    break;
  default:
    printf ("\tValeur du champ 'sous_image' impossible !!!\n");
    break;
  }

  /* tat des donnes : les points...  */
  switch (im3d->header.points) {
  case NIL_POINTS:
    printf ("\t(Tableau de points non allou)\n");
    break;
  case POINTS_ALLOUES:
    printf ("\tTableau de points seulement allou (%d lignes, %d colonnes)\n",
	    im3d->header.nblig, im3d->header.nbcol);
    break;
  case POINTS_CALCULES:
    printf ("\tTableau de points allou et dtermin (%d lignes, %d colonnes)\n",
	    im3d->header.nblig, im3d->header.nbcol);
    break;
  }

  /* ... les covariances ...*/
  switch (im3d->header.covariances) {
  case NIL_COVARIANCES:
    printf ("\t(Tableau des covariances non allou)\n");
    break;
  case COVARIANCES_ALLOUEES:
    printf ("\tTableau des covariances seulement allou\n");
    break;
  case COVARIANCES_CALCULEES:
    printf ("\tTableau des covariances dtermin\n");
    break;
  }

  /* ... les normales... */
  switch (im3d->header.normales) {
  case NIL_NORMALES:
    printf ("\t(Tableau des normales non allou)\n");
    break;
  case NORMALES_ALLOUEES:
    printf ("\tTableau des normales seulement allou\n");
    break;
  case NORMALES_CALCULEES:
    printf ("\tTableau des normales dtermin ");
    switch (im3d->header.nb_voisins_normales) {
    case QUATRE_VOISINS:
      printf (" partir de 4 voisins\n");
      break;
    case HUIT_VOISINS:
      printf (" partir de 8 voisins\n");
      break;
    default:
      printf ("??? Pb champ 'type_normales'\n");
      break;
    }
    break;
  }

  /* ... et les infos locales */
  switch (im3d->header.local) {
  case NIL_INFOS_LOCALES:
    printf ("\t(Pas d'infos locales alloues)\n");
    break;
  case INFOS_LOCALES_ALLOUEES:
    printf ("\tTableau des infos locales seulement allou (taille %d octets)\n",
	    im3d->header.size_local);
    break;
  case INFOS_LOCALES_CALCULEES:
    printf ("\tTableau des infos locales dtermin (taille %d octets)\n",
	    im3d->header.size_local);
    break;
  }

  /* Valeurs max */
  if (im3d->header.bornes == BORNES_CALCULEES) {
    switch (im3d->header.coordonnees) {

    case COORDONNEES_CARTESIENNES:
      printf ("\tBornes sur les coordonnes :\n");
      printf ("\t\t%.2f < X < %.2f\n",
	      im3d->header.min_coord_1, im3d->header.max_coord_1);
      printf ("\t\t%.2f < Y < %.2f\n",
	      im3d->header.min_coord_2, im3d->header.max_coord_2);
      printf ("\t\t%.2f < Z < %.2f\n",
	      im3d->header.min_coord_3, im3d->header.max_coord_3);
      break;

    case COORDONNEES_CYLINDRIQUES:
      printf ("\tBornes sur les coordonnes :\n");
      printf ("\t\t%.2f < Rayon < %.2f\n",
	      im3d->header.min_coord_1, im3d->header.max_coord_1);
      printf ("\t\t%.2f < Theta < %.2f\n",
	      im3d->header.min_coord_2 * 180./M_PI, im3d->header.max_coord_2 * 180./M_PI);
      printf ("\t\t%.2f < Z < %.2f\n",
	      im3d->header.min_coord_3, im3d->header.max_coord_3);
      break;

    case COORDONNEES_SPHERIQUES:
      printf ("\tBornes sur les coordonnes :\n");
      printf ("\t\t%.2f < Rayon < %.2f\n",
	      im3d->header.min_coord_1, im3d->header.max_coord_1);
      printf ("\t\t%.2f < Theta < %.2f\n",
	      im3d->header.min_coord_2 * 180./M_PI, im3d->header.max_coord_2 * 180./M_PI);
      printf ("\t\t%.2f < Phi < %.2f\n",
	      im3d->header.min_coord_3 * 180./M_PI, im3d->header.max_coord_3 * 180./M_PI);
      break;

    default:
      printf ("\tSystme de coordonnes INCONNU !!!\n");
      printf ("\t\t%.2f < 1re coordonne < %.2f\n",
	      im3d->header.min_coord_1, im3d->header.max_coord_1);
      printf ("\t\t%.2f < 2nde coordonne < %.2f\n",
	      im3d->header.min_coord_2, im3d->header.max_coord_2);
      printf ("\t\t%.2f < 3me coordonne < %.2f\n",
	      im3d->header.min_coord_3, im3d->header.max_coord_3);
      break;
    }
  }
  else
    printf ("\t(Valeurs extrmales non dtermines)\n");

  printf ("\n");
  return (ERR_IM3D_OK);
}



/**********************************************************************
 *
 * Affiche  l'cran tout ce qu'on sait sur le point 3D
 *
 **********************************************************************/

int info_im3d_pt3d (DATA_IM3D*  im3d,
			  int         i,
			  int         j)

{
  DATA_PT3D*      pt3d;

  if (im3d->header.points == NIL_POINTS)
    return (ERR_IM3D_POINTS_NON_ALLOUES);

  if ((i < 0) || (i >= im3d->header.nblig) || (j < 0) || (j >= im3d->header.nbcol)) {
    printf ("Eh, il faut respecter 0 <= i < %d et 0 <= j < %d !\n",
	    im3d->header.nblig, im3d->header.nbcol);
    return (ERR_IM3D_INDICES);
  }

  pt3d = im3d->points_tab + (im3d->header.nbcol * i) + j;

  if (pt3d->state == BAD_PT3D) {
    printf ("\tPoint (%d, %d) non valide\n", i, j);
    return (ERR_IM3D_OK);
  }

  info_data_pt3d (pt3d);

  return (ERR_IM3D_OK);
}



