/*	$Id: io.c 2007/11/29 10:33:15 mallet $ */

/*
 * Copyright (c) 1997 LAAS/CNRS                       --  Thu Aug  3 2006
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

#include <string.h>

#include "libimages3d.h"

static int	write_im3d_header(FILE *fichier,
				  DATA_IM3D_HEADER *im3d_header);


/**********************************************************************
 *
 * Ecriture sur fichier d'une structure 'DATA_IM3D' en ASCII. On ne sauve pas
 * les infos locales (ni les flags correspondants), vu qu'on ne sait pas
 * vraiment ce qu'il y a dedans. Peut-tre le fera-t-on pour les sauvegarde
 * binaires ? (en connaissant la taille des infos locales, c'est possible).
 *
 * Remarque : on peut avec cette fct sauvegarder une image vide (je sais, a
 * n'a pas de sens !)
 *
 **********************************************************************/

int write_data_im3d_ascii (DATA_IM3D*  im3d,
			   char*       nom_fichier)
{
  FILE *fichier;
  int          nblig;
  int          nbcol;
  int          i,j,k;		/* Pour boucler */
  DATA_PT3D*   point;

  /* On sauve dans le cwd */
  fichier = fopen(nom_fichier, "w9");

  if (fichier == NULL)
    return (ERR_IM3D_OPEN_ASCII_FILE);


  /* Ok. Maintenant on va sauvegarder tout a */
  /* le mot clef */
  fprintf (fichier, FF_DATA_IM3D_ASCII);
  fprintf (fichier,"\n");

  /* L'entte de l'image */
  write_im3d_header (fichier, &(im3d->header));

  nblig = im3d->header.nblig;
  nbcol = im3d->header.nbcol;

  /* C'est parti pour les points (les covariances et normales sont ecrites apres) */
  if (im3d->header.points == POINTS_CALCULES) {
    point = im3d->points_tab;
    for (i = 0; i < nblig; i++) {
      for (j = 0; j < nbcol; j++) {
	/* Les diffrents flags */
	fprintf (fichier,"%d %d %d %d %d %d ",
		      (int) point->state, (int) point->repere,
		      (int) point->coordonnees, (int) point->normales,
		      (int) point->nb_voisins_normales,
		      (int) point->covariances);
	/* Les coordonnes */
	fprintf (fichier,"%f %f %f ",
		      point->coord_1, point->coord_2,	point->coord_3);
	point++;
      }
      /* Retour chariot au changement de ligne */
      fprintf (fichier,"\n");
    }
  }
  fprintf (fichier,"\n");

  /* C'est parti pour les matrices de covariance */
  if (im3d->header.covariances == COVARIANCES_CALCULEES) {
    point = im3d->points_tab;
    for (i = 0; i < nblig; i++) {
      for (j = 0; j < nbcol; j++) {
	for (k = 0; k < IM3D_COVARIANCES_SIZE; k++)
	  fprintf (fichier,"%f ",point->covariances_tab[k]);
	point++;
      }
      /* Retour chariot au changement de ligne */
      fprintf (fichier,"\n");
    }
  }
  fprintf (fichier,"\n");

  /* C'est parti pour les normales ... */
  if (im3d->header.normales == NORMALES_CALCULEES) {
    point = im3d->points_tab;
    for (i = 0; i < nblig; i++) {
      for (j = 0; j < nbcol; j++) {
	for (k = 0; k < IM3D_NORMALES_SIZE; k++)
	  fprintf (fichier,"%f ",point->normales_tab[k]);
	point++;
      }
      /* Retour chariot au changement de ligne */
      fprintf (fichier,"\n");
    }
  }
  fprintf (fichier,"\n");


  /* That's it, on ferme la porte et on met les bouts */
  fclose(fichier);

  return (ERR_IM3D_OK);
}

/**********************************************************************
 *
 * Ecriture sur un fichier de l'entte d'une structure 'DATA_IM3D' (toujours
 * en ASCII)
 *
 **********************************************************************/

static int
write_im3d_header(FILE *fichier, DATA_IM3D_HEADER*  im3d_header)
{
  int i;

  /* On commence par les caracteristiques gnrales de l'image */
  fprintf (fichier,"%d %d %d %d %d %d\n",
	  (int) im3d_header->robot, (int) im3d_header->capteur,
	  (int) im3d_header->repere, (int) im3d_header->coordonnees,
	  (int) im3d_header->reduction, (int) im3d_header->sous_image);

  /* Infos sur les champs des points */
  fprintf (fichier,"%d %d %d %d %d %d\n",
	  (int) im3d_header->points, (int) im3d_header->normales,
	  (int) im3d_header->nb_voisins_normales, (int) im3d_header->covariances,
	  (int) im3d_header->local, (int) im3d_header->bornes);

  /* Infos sur la taille de l'image */
  fprintf (fichier,"%d %d %d %d %d %d %d %d %d\n",
	  im3d_header->nblig_initial, im3d_header->nbcol_initial,
	  im3d_header->nblig, im3d_header->nbcol,
	  im3d_header->i_min, im3d_header->i_max,
	  im3d_header->j_min, im3d_header->j_max,
	  im3d_header->taux_reduction);

  /* Matrice intrinseque */
  for(i = 0; i < 16; i++)
    fprintf (fichier, "%f ", im3d_header->mat_intrinseque[i]);
  fprintf (fichier, "\n");

  /* La Tr3d tr_capteur_2_repere : on sauve les angles d'euler */
  t3dConvertTo(T3D_BRYAN, &im3d_header->tr_capteur_2_repere);
  fprintf (fichier, "%f %f %f %f %f %f\n",
	    (im3d_header->tr_capteur_2_repere).euler.euler[0],
	    (im3d_header->tr_capteur_2_repere).euler.euler[1],
	    (im3d_header->tr_capteur_2_repere).euler.euler[2],
	    (im3d_header->tr_capteur_2_repere).euler.euler[3],
	    (im3d_header->tr_capteur_2_repere).euler.euler[4],
	    (im3d_header->tr_capteur_2_repere).euler.euler[5]);
/*   fwrite(fichier, &im3d_header->tr_capteur_2_repere, sizeof(T3D)); */

  /* La taille du champ local : on ne l'imprime pas */
  /*   fprintf (fichier, "%d\n", im3d_header->size_local); */

  /* Min - Max -- c'est peut etre pas la peine ... bof, allez :-D */
  if (im3d_header->bornes == BORNES_CALCULEES) {
    fprintf (fichier,"%f %f %f %f %f %f\n",
	    im3d_header->max_coord_1, im3d_header->max_coord_2,
	    im3d_header->max_coord_3, im3d_header->min_coord_1,
	    im3d_header->min_coord_2, im3d_header->min_coord_3);
  }

  fprintf (fichier, "\n\n");

  return (ERR_IM3D_OK);
}


/**********************************************************************
 *
 * Lecture  partir d'un fichier ASCII d'une structure 'DATA_IM3D'.
 *
 **********************************************************************/

int read_data_im3d_ascii (DATA_IM3D*  im3d,
				char*       nom_fichier)

{
  char		entete[256];
  int		i, j, k;
  int		nblig, nbcol;
  FILE *	fichier;	/* Ben le fichier qu'on lit */
  int		tmp;		/* Petite variable intermdiaire */
  DATA_PT3D*	point;
  int		flag1, flag2, flag3, flag4, flag5, flag6;
  int		s;

  /* Ouverture du fichier */
  fichier = fopen(nom_fichier, "r");

  if (fichier == NULL)
    return (ERR_IM3D_OPEN_ASCII_FILE);

  fscanf (fichier, "%s", entete);
  if (strcmp (entete, FF_DATA_IM3D_ASCII) != 0) {
    printf ("*** Le ficher n'est pas au bon format (%s) ...\n", FF_DATA_IM3D_ASCII);
    fclose (fichier);
    return (ERR_IM3D_FORMAT_ASCII_FILE);
  }

  /* Lecture de l'entte de l'image 3D */
  s = read_im3d_header (fichier, &(im3d->header));
  if (s != ERR_IM3D_OK) {
    fclose (fichier);
    return s;
  }

  /* On rcupre les variables dont on a besoin */
  nblig = im3d->header.nblig;
  nbcol = im3d->header.nbcol;

  /* On commence par faire les allocations de tous les tableaux. A chaque
     fois, on positionne le flag correspondant  NIL pour pouvoir appeler les
     fct d'allocation, (on mmorise la valeur relle et on la repositionne
     aprs) */
  /* Les points */
  if (im3d->header.points != NIL_POINTS) {
    tmp = (int) im3d->header.points;
    im3d->header.points = NIL_POINTS;

    s = alloue_im3d_points (im3d,
			    im3d->header.nblig, im3d->header.nbcol);
    if (s != ERR_IM3D_OK) {
      fclose(fichier);
      return s;
    }
    im3d->header.points = tmp;
  }
  /* Les covariances */
  if (im3d->header.covariances != NIL_COVARIANCES) {
    tmp = (int) im3d->header.covariances;
    im3d->header.covariances = NIL_COVARIANCES;

    s = alloue_im3d_covariances(im3d);
    if (s != ERR_IM3D_OK) {
      fclose(fichier);
      return s;
    }
    im3d->header.covariances = tmp;
  }
  /* Les normales */
  if (im3d->header.normales != NIL_NORMALES) {
    tmp = (int) im3d->header.normales;
    im3d->header.normales = NIL_NORMALES;

    s = alloue_im3d_normales (im3d);
    if (s != ERR_IM3D_OK) {
      fclose(fichier);
      return s;
    }
    im3d->header.normales = tmp;
  }

  /* On lit les points. */
  if (im3d->header.points == POINTS_CALCULES) {
    point = im3d->points_tab;
    for (i = 0; i < nblig; i++) {
      for (j = 0; j < nbcol; j++) {
	/* Les diffrents flags. On les lit un par un. */
	if (fscanf (fichier,"%d %d %d %d %d %d ",
		    &flag1, &flag2, &flag3, &flag4, &flag5, &flag6) != 6) {
	  fclose(fichier);
	  return (ERR_IM3D_READ_ASCII_FILE);
	}
	point->state = flag1;
	point->repere = flag2;
	point->coordonnees = flag3;
	point->normales = flag4;
	point->nb_voisins_normales = flag5;
	point->covariances = flag6;

	/* Les coordonnes */
	if (fscanf (fichier, "%f %f %f ",
		    &(point->coord_1), &(point->coord_2), &(point->coord_3)) != 3) {
	  fclose(fichier);
	  return  (ERR_IM3D_READ_ASCII_FILE);
	}
	point++;
      }
      /* Retour chariot au changement de ligne */
      fscanf (fichier,"\n");
    }
  } /* Fin de lecture des points */
  fscanf (fichier,"\n");



  if (im3d->header.covariances == COVARIANCES_CALCULEES) {
    /* On lit les covariances. */
    point = im3d->points_tab;
    for (i = 0; i < nblig; i++) {
      for (j = 0; j < nbcol; j++) {
	for (k = 0; k < IM3D_COVARIANCES_SIZE; k++)
	  if (fscanf (fichier, "%f ", point->covariances_tab + k) != 1) {
	    fclose(fichier);
	    return (ERR_IM3D_READ_ASCII_FILE);
	  }
	point++;
      }
      /* Retour chariot au changement de ligne */
      fscanf (fichier, "\n");
    }
  } /* Fin de lecture des covariances */
  fscanf (fichier, "\n");


  /* On lit les normales. */
  if (im3d->header.normales == NORMALES_CALCULEES) {
    point = im3d->points_tab;
    for (i = 0; i < nblig; i++) {
      for (j = 0; j < nbcol; j++) {
	for (k = 0; k < IM3D_NORMALES_SIZE; k++)
	  if (fscanf (fichier, "%f ", point->normales_tab + k) != 1) {
	    fclose(fichier);
	    return (ERR_IM3D_READ_ASCII_FILE);
	  }
	point++;
      }
      /* Retour chariot au changement de ligne */
      fscanf (fichier, "\n");
    }
  }
  fscanf (fichier,"\n");


  /* On ferme le fichier et on s'en va */
  fclose(fichier);

  printf("Im3d read successfully !\n");

  return (ERR_IM3D_OK);
}


/**********************************************************************
 *
 * Lecture sur un fichier de l'entte d'une structure 'DATA_IM3D' (toujours
 * en ASCII)
 *
 **********************************************************************/

int read_im3d_header (FILE*		fichier,
			    DATA_IM3D_HEADER*	im3d_header)

{
  int i;

  /* On commence par les caracteristiques gnrales de l'image */
  if (fscanf (fichier, "%d %d %d %d %d %d\n",
	      (int*) &(im3d_header->robot),
	      (int*) &(im3d_header->capteur),
	      (int*) &(im3d_header->repere),
	      (int*) &(im3d_header->coordonnees),
	      (int*) &(im3d_header->reduction),
	      (int*) &(im3d_header->sous_image)) != 6)
    return  (ERR_IM3D_READ_HEADER);

  /* Infos sur les champs des points */
  if (fscanf (fichier, "%d %d %d %d %d %d\n",
	      (int*) &(im3d_header->points),
	      (int*) &(im3d_header->normales),
	      (int*) &(im3d_header->nb_voisins_normales),
	      (int*) &(im3d_header->covariances),
	      (int*) &(im3d_header->local),
	      (int*) &(im3d_header->bornes)) != 6)
    return  (ERR_IM3D_READ_HEADER);

  /* Infos sur la taille de l'image */
  if (fscanf (fichier, "%d %d %d %d %d %d %d %d %d\n",
	      &(im3d_header->nblig_initial), &(im3d_header->nbcol_initial),
	      &(im3d_header->nblig), &(im3d_header->nbcol),
	      &(im3d_header->i_min), &(im3d_header->i_max),
	      &(im3d_header->j_min), &(im3d_header->j_max),
	      &(im3d_header->taux_reduction)) != 9)
    return  (ERR_IM3D_READ_HEADER);

  /* Matrice intrinseque */
  for (i = 0; i < 16; i++)
    if (fscanf (fichier, "%lf ", (im3d_header->mat_intrinseque + i)) != 1)
      return  (ERR_IM3D_READ_HEADER);

  fscanf (fichier, "\n");

  /* La Tr3d tr_capteur_2_repere */
  im3d_header->tr_capteur_2_repere = t3dBryanIdentity;
  if (fscanf (fichier, "%f %f %f %f %f %f\n",
	      &(im3d_header->tr_capteur_2_repere).euler.euler[0],
	      &(im3d_header->tr_capteur_2_repere).euler.euler[1],
	      &(im3d_header->tr_capteur_2_repere).euler.euler[2],
	      &(im3d_header->tr_capteur_2_repere).euler.euler[3],
	      &(im3d_header->tr_capteur_2_repere).euler.euler[4],
	      &(im3d_header->tr_capteur_2_repere).euler.euler[5]) != 6) 
      return  (ERR_IM3D_READ_HEADER);
/*   fread(&im3d_header->tr_capteur_2_repere, sizeof(T3D), 1, fichier); */

  /* La taille du champ local */
  /*   if (fscanf (fichier, "%d\n", &(im3d_header->size_local)) != 1) */
  /*     return  (ERR_IM3D_READ_HEADER); */

  /* Min - Max -- c'est peut etre pas la peine ... bof, allez :-D */
  if (im3d_header->bornes == BORNES_CALCULEES) {
    if (fscanf (fichier, "%f %f %f %f %f %f\n",
		&(im3d_header->max_coord_1),
		&(im3d_header->max_coord_2),
		&(im3d_header->max_coord_3),
		&(im3d_header->min_coord_1),
		&(im3d_header->min_coord_2),
		&(im3d_header->min_coord_3)) != 6)
      return  (ERR_IM3D_READ_HEADER);
  }

  fscanf (fichier, "\n\n");

  return (ERR_IM3D_OK);
}


/**********************************************************************
 *
 * Ecriture sur fichier d'une structure 'DATA_IM3D' en biainre (sauf le
 * header, qui reste en ASCII) . On sauve pas les infos locales et les
 * flags correspondants.
 *
 * Remarque : on peut avec cette fct sauvegarder une image vide (je sais, a
 * n'a pas de sens !)
 *
 **********************************************************************/

int write_data_im3d_binaire (DATA_IM3D*  im3d,
			     char*       nom_fichier)

{
  FILE *fichier;
  int          nblig;
  int          nbcol;
  int          i,j,k;		/* Pour boucler */
  DATA_PT3D*   point;

  /* On sauve dans le cwd */
  fichier = fopen(nom_fichier, "w9");

  if (fichier == NULL)
    return (ERR_IM3D_OPEN_ASCII_FILE);


  /* Ok. Maintenant on va sauvegarder tout a */
  /* le mot clef */
  fprintf (fichier, FF_DATA_IM3D_BINARY);
  fprintf (fichier,"\n");

  /* L'entte de l'image */
  write_im3d_header (fichier, &(im3d->header));

  nblig = im3d->header.nblig;
  nbcol = im3d->header.nbcol;

  /* C'est parti pour les points (les covariances et normales sont ecrites apres) */
  if (im3d->header.points == POINTS_CALCULES) {
    point = im3d->points_tab;
    for (i = 0; i < nblig; i++) {
      for (j = 0; j < nbcol; j++) {
	fwrite (point, sizeof(DATA_PT3D), 1, fichier);
	point++;
      }
    }
  }

  /* That's it, on ferme la porte et on met les bouts */
  fclose(fichier);

  return (ERR_IM3D_OK);
}

/**********************************************************************
 *
 * Lecture  partir d'un fichier binaire d'une structure 'DATA_IM3D'.
 *
 **********************************************************************/

int
read_data_im3d_binaire (DATA_IM3D*  im3d,
			char*       nom_fichier)

{
  char		entete[256];
  FILE*		fichier;
  int		nblig;
  int		nbcol;
  int		i,j,k;		/* Pour boucler */
  DATA_PT3D*	point;
  int		s;
  int		tmp;

  /* On sauve dans le cwd */
  fichier = fopen(nom_fichier, "r");

  if (fichier == NULL)
    return (ERR_IM3D_OPEN_ASCII_FILE);

  /* le mot clef */
  fscanf (fichier, "%s", entete);
  if (strcmp (entete, FF_DATA_IM3D_BINARY) != 0) {
    printf ("*** Le ficher n'est pas au bon format (%s) ...\n", 
	    FF_DATA_IM3D_BINARY);
    fclose (fichier);
    return (ERR_IM3D_FORMAT_ASCII_FILE);
  }

  /* L'entte de l'image */
  s = read_im3d_header (fichier, &(im3d->header));
  if (s != ERR_IM3D_OK) {
    fclose (fichier);
    return s;
  }

  /* On rcupre les variables dont on a besoin */
  nblig = im3d->header.nblig;
  nbcol = im3d->header.nbcol;
  printf ("%d %d\n", nblig, nbcol);

  /* Allocation du tableau des points */
  if (im3d->header.points != NIL_POINTS) {
    tmp = (int) im3d->header.points;
    im3d->header.points = NIL_POINTS;
    s = alloue_im3d_points (im3d,
			    im3d->header.nblig, im3d->header.nbcol);
    if (s != ERR_IM3D_OK) {
      fclose(fichier);
      return s;
    }
    im3d->header.points = tmp;
  }

  /* C'est parti pour les points (les covariances et normales sont ecrites apres) */
  if (im3d->header.points == POINTS_CALCULES) {
    point = im3d->points_tab;
    for (i = 0; i < nblig; i++) {
      for (j = 0; j < nbcol; j++) {
	if (fread (point, sizeof(DATA_PT3D), 1, fichier) != 1)
	  return (ERR_IM3D_READ_ASCII_FILE);
	point++;
      }
    }
  }

  /* That's it, on ferme la porte et on met les bouts */
  fclose(fichier);

  return (ERR_IM3D_OK);
}


